#!/usr/bin/perl

############################### This program is free software; you can redistribute it and/or modify
############################### it under the terms of the GNU General Public License as published by
############################### the Free Software Foundation; either version 2 of the License, or
############################### (at your option) any later version.
############################### 
############################### This program is distributed in the hope that it will be useful,
############################### but WITHOUT ANY WARRANTY; without even the implied warranty of
############################### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
############################### GNU General Public License for more details.
############################### 
############################### You should have received a copy of the GNU General Public License
############################### along with this program; if not, write to the Free Software
############################### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
###############################
###############################
###############################
############################### Copyright 2006 Damon P. Little


############################### STANDARD INFO
use DBI;
my $db = "barcode";
my $host = "localhost";
my $id = "barcode";
my $pwd = "barcode";
$loci = "loci";
$sequences = "sequences";
$taxa = "taxa";
$motifs = "motif";
$window = 10;

############################### GET OPTIONS
if($ARGV[0] eq "-e"){
	$option = "=";
	} elsif ($ARGV[0] eq "-l"){
		$option = "LIKE";
		} 
if($ARGV[1]){
	$locus = $ARGV[1]; 
	} else {
		$locus = "NULL";
		}
if($ARGV[2]){
	$number = $ARGV[2]
	} else {
		$number = "-a";
		}
		
		
		
############################### ERROR TRAP
if($locus eq "NULL"){
	print "A Perl script for finding unique barcodes.\n";
	print "USAGE: barcode.pl -e[-l] locus -a[-h]\n";
	print "where -e specifies exact locus match; -l LIKE\n";
	print "-a all sequences; -h high scoring sequences\n";
	} else {
		print "\n";
		
		############################### CONNECT TO DB
		my $dbh = DBI->connect("DBI:mysql:$db:$host",$id,$pwd);
		if(!defined $dbh){
			die "Can't connect: $DBI::errstr \n";
			}
		
		############################### QUERY
		$mystr = "SELECT $taxa.taxon_id, $taxa.genus, $taxa.species, $taxa.subsp, $loci.locus_id, $sequences.sequence 
				  FROM $sequences, $taxa, $loci 
				  WHERE $taxa.taxon_id=$sequences.taxon_id AND $sequences.locus_id=$loci.locus_id 
				  AND $loci.name_short $option ?
				  ORDER BY $taxa.genus, $taxa.species, $sequences.score DESC";	
		$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
		$rv = $sth->execute ($locus) || die "can't execute the query: $sth->errstr";
		my $table = $sth->fetchall_arrayref || die "$sth->errstr\n";
		
		$last_genus = "";
		$last_species = "";
		$last_seq = "";
		$buffer = "<HTML><BODY><TABLE>\n<TR><TD>SPECIES</TD><TD>NUMBER OF UNIQUE BARCODES</TD></TR>\n";
		$zero_count = 0;
		for $k (0 .. $#{$table}){					
			$taxon_id = $table->[$k][0];
			$genus = $table->[$k][1];
			$species = $table->[$k][2];
			$subsp = $table->[$k][3];
			$locus_id = $table->[$k][4];
			$seq = $table->[$k][5];
		
			if((($genus eq $last_genus) && ($species eq $last_species) && ($number eq "-a") && ($seq ne $last_seq)) || ($genus ne $last_genus) || ($species ne $last_species)){
				
				############################### GET NUMBER OF SEQUENCES
				$mystr = "SELECT COUNT(*)
						  FROM $sequences, $taxa, $loci 
						  WHERE $taxa.taxon_id=$sequences.taxon_id AND $sequences.locus_id=$loci.locus_id 
						  AND $loci.name_short $option ?
						  AND $taxa.genus = ? 
						  AND $taxa.species = ?";	
				$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
				$rv = $sth->execute ($locus, $genus, $species) || die "can't execute the query: $sth->errstr";
				my $table = $sth->fetchall_arrayref || die "$sth->errstr\n";
				$count = $table->[0][0];
					
				############################### BREAK UP SEQUENCE HALF OVERLAP
				@junk = split("", $seq);
				@motif = "";
				$space = $window / 2;
				for($j = @junk; $j >= 0; ($j -= $space)){
					$start = $j - $window + 1;
					$end = $j;
					if(($start >= 0) && ($end < @junk)){
						push(@motif, join("", @junk[$start..$end]));										
						}
					}
				shift(@motif);
				
				############################### QUERY
				$insert = 0;
				foreach $value(@motif){
					print "\r barcode.pl |";
					if(500 > $insert){	
						$revcom = reverse($value);
						$revcom =~ tr/ACGTNVDBHWMRKSY/TGCANBHVDWKYMSR/;
					
						############################### FIND MOTIF IN OTHER
						$mystr = "SELECT COUNT(*)
								  FROM $sequences, $taxa, $loci 
								  WHERE $taxa.taxon_id=$sequences.taxon_id AND $sequences.locus_id=$loci.locus_id 
								  AND $loci.name_short $option ?
								  AND ($sequences.sequence LIKE \"\%$value\%\" OR $sequences.sequence LIKE \"\%$revcom\%\")
								  AND ($taxa.genus != ? OR ($taxa.genus = ? AND $taxa.species != ?))";	
						$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
						$rv = $sth->execute ($locus, $genus, $genus, $species) || die "can't execute the query: $sth->errstr";
						my $table = $sth->fetchall_arrayref || die "$sth->errstr\n";
						$motif_count = $table->[0][0];
						
						print "\r barcode.pl -";					
						############################### FIND MOTIF IN SAME
						if($mo_count == 0){
							$mystr = "SELECT COUNT(*)
									  FROM $sequences, $taxa, $loci 
									  WHERE $taxa.taxon_id=$sequences.taxon_id AND $sequences.locus_id=$loci.locus_id 
									  AND $loci.name_short $option ?
									  AND ($sequences.sequence LIKE \"\%$value\%\" OR $sequences.sequence LIKE \"\%$revcom\%\")
									  AND $taxa.genus = ? 
									  AND $taxa.species = ?";	
							$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
							$rv = $sth->execute ($locus, $genus, $species) || die "can't execute the query: $sth->errstr";
							my $table = $sth->fetchall_arrayref || die "$sth->errstr\n";
							$other_count = $table->[0][0];
							}
						
						print "\r barcode.pl \\";
						############################### INSERT NEW RECORD
						if(($motif_count == 0) && ($other_count == $count)){#if motif is unique and found in all of group
							$insert++;
							$mystr = "INSERT IGNORE INTO $motifs (motif_id, taxon_id, motif, locus_id) VALUES (NULL, ?, ?, ?)";
							$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
							$rv = $sth->execute ($taxon_id, $value, $locus_id) || die "can't execute the query: $sth->errstr";						
							}
						}
					}	
				############################### OUTPUT
				$buffer .= "<TR><TD>" .$genus . " " . $species . "</TD><TD>" . $insert . "</TD></TR>\n";
				if($insert == 0){
					$zero_count++;
					}
				print "\r barcode.pl /";
				
				}
			$last_genus = $genus;
			$last_species = $species;
			$last_seq = $seq;		
			}
						
		############################### FINISH
		$rc = $sth->finish;
		$dbh->disconnect;
		}

$buffer .= "</TABLE><BR>";
$buffer .= "$zero_count taxa without barcodes<BR></BODY></HTML>\n";
print "\r$buffer\n";
exit;
