#!/usr/bin/perl

############################### This program is free software; you can redistribute it and/or modify
############################### it under the terms of the GNU General Public License as published by
############################### the Free Software Foundation; either version 2 of the License, or
############################### (at your option) any later version.
############################### 
############################### This program is distributed in the hope that it will be useful,
############################### but WITHOUT ANY WARRANTY; without even the implied warranty of
############################### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
############################### GNU General Public License for more details.
############################### 
############################### You should have received a copy of the GNU General Public License
############################### along with this program; if not, write to the Free Software
############################### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
###############################
###############################
###############################
############################### Copyright 2006 Damon P. Little

if(!$ARGV[0]){
	print "A Perl script for identifying an unknown sequence based on barcodes\n";
	print "USAGE: dome.pl sequence\n";
	print "sequence is assumed to be a single string read from stdin\n";
	} else {
		############################### STANDARD INFO
		use DBI;
		my $db = "barcode";
		my $host = "localhost";
		my $id = "barcode";
		my $pwd = "barcode";
		$sequences = "sequences";
		$taxa = "taxa";
		$motif = "motif";
		
		
		
		############################### CONNECT TO DB
		my $dbh = DBI->connect("DBI:mysql:$db:$host",$id,$pwd);
		if(!defined $dbh){
			die "Can't connect: $DBI::errstr \n";
			}
		
		
		
		############################### FIND MOTIFS FORWARD
		$sequence = uc($ARGV[0]);
		$sequence  =~ tr/ACGTNVDBHWMRKSY//cd;
		@junk = split("", $sequence);
		$inc = 10;
		@motifs = "";
		for($k = @junk; $k >= 0; $k--){
			$start = $k - $inc + 1;
			$end = $k;
			if(($start >= 0) && ($end < @junk)){
				push(@motifs, join("", @junk[$start..$end]));
				}
			}
		
			
			
		############################### FIND MOTIFS BACKWARDS	
		$revcom = reverse($sequence);
		$revcom =~ tr/ACGTNVDBHWMRKSY/TGCANBHVDWKYMSR/;
		@junk = split("", $revcom);
		for($k = @junk; $k >= 0; $k--){
			$start = $k - $inc + 1;
			$end = $k;
			if(($start >= 0) && ($end < @junk)){
				push(@motifs, join("", @junk[$start..$end]));										
				}
			}
		
		
		
		############################### UNIQUE ONLY
		shift(@motifs);
		undef %saw;
		@out = grep(!$saw{$_}++, @motifs);
		@motifs = @out;
		
		
		
		############################### BUILD QUERY	
		$query = "	SELECT $taxa.taxon_id, $taxa.genus, $taxa.species, count(*) AS hits
					FROM $motif, $taxa 
					WHERE $motif.taxon_id = $taxa.taxon_id
					AND (";
		$j = 0;			
		foreach $value (@motifs){
			if($j != 0){
				$query .= " OR ";
				}
			$query .= "$motif.motif = \"$value\"";		
			$j++;			
			}
		$query .=	") 
					GROUP BY $taxa.taxon_id 
					ORDER BY hits DESC, $taxa.genus, $taxa.species";				
		$sth = $dbh->prepare($query) || die "Can't prepare $statement: $dbh->errstr\n";
		$rv = $sth->execute || die "can't execute the query: $sth->errstr";
		my $table = $sth->fetchall_arrayref || die "$sth->errstr\n";
		
		
		
		############################### PRINT PRETTY
		$buffer = "";
		$i = 0;
		for $k (0 .. $#{$table}){					
			$genus = $table->[$k][1];
			$species = $table->[$k][2];
			$hits = $table->[$k][3];
			$buffer .= "$genus $species hits = $hits\n";
			$i++;
			}
		
			
			
		############################### FINISH
		$rc = $sth->finish;
		$dbh->disconnect;	
		
		if($i == 0){
			print "Sorry, no identification could be made.\n";
			} else {
				print "$buffer";
				}
		}

exit;
